/* Game.cpp
	This class contains all the information about the game world (terrain, entity grid). However, its
	larger purpose is as a manager of Entities, Particles, Renderers, etc. It handles the creation of 
	new Entities and the cleanup of killed enemies. 
	The class is independant of any timing, and assumes a fixed delta time provided by the class calling
	its update/render functions.
	The game class is a singleton, and is globally accessable with a single entry point (Game::Instance())
	The class is also the place where helper functions such as DirToVec2 are placed for this example.
*/

#include "GFX.h"
#include "Core.h"
#include "Entity.h"
#include "SDL.h"
#include "SDL_opengl.h"
#include "SDL_TTF.h"

//Screen attributes
const int SCREEN_WIDTH = 1024;
const int SCREEN_HEIGHT = 768;
const int SCREEN_BPP = 32;

//The frame rate
const int FRAMES_PER_SECOND = 60;

//The attributes of the square
const int SQUARE_WIDTH = 20;
const int SQUARE_HEIGHT = 20;

SDL_Event event;

Core* Core::instance = 0;

Core* Core::Instance() {
	if(instance == 0) {
		instance = new Core();
	}
	return instance;
}

Core::Core() : RenderManager(), CollisionManager(){
	tracking = 0;
	cameraZoom = 1;
}

// Don't optimize ticks
volatile int ticks;

// Timing callback
void ticker()
{
        ticks++;
}
#ifdef BUILD_ALLEGRO
END_OF_FUNCTION(ticker)
#endif 

bool Core::Init() {

    //Initialize SDL
    if( SDL_Init( SDL_INIT_EVERYTHING ) < 0 )
    {
        return false;
    }
	screenWidth = SCREEN_WIDTH;
	screenHeight = SCREEN_HEIGHT;
	gfx = GFX::Instance();
	
	gfx->screen = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_OPENGL );
    //Create Window
	if(!gfx->screen)
    {
        return false;
    }

	//Initialize SDL_ttf
    if( TTF_Init() == -1 )
    {
        return false;
    }

    //Initialize OpenGL
    if( init_GL() == false )
    {
        return false;
    }

    //Set caption
    SDL_WM_SetCaption( "OpenGL Test", NULL );
	paused = false;

    return true;

}


bool Core::init_GL()
{
    //Set clear color
    glClearColor( 0, 0, 0, 0 );

    //Set projection
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    glOrtho( 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -1, 1 );

    //Initialize modelview matrix
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    //If there was any errors
    if( glGetError() != GL_NO_ERROR )
    {
        return false;
    }

    //If everything initialized
    return true;
}

void Core::SetCamera(Vec2 newpos) {
	cameraCenter = newpos;
	glTranslatef(-cameraCenter.x, -cameraCenter.y, 0);
}

Vec2 Core::GetUpperClip() {
	return cameraCenter + Vec2(screenWidth / cameraZoom, screenHeight / cameraZoom);
}

Vec2 Core::GetLowerClip() {
	return cameraCenter;
}

void Core::TogglePause() {
	paused = !paused;
}

bool Core::InCameraRange(float x1, float y1, float x2, float y2) {
	Vec2 lowerClip = GetLowerClip();
	Vec2 upperClip = GetUpperClip();
	if(x2 < x1) { float temp = x2; x2 = x1; x1 = temp; }
	if(y2 < y1) { float temp = y2; y2 = y1; y1 = temp; }
	if(x2 < lowerClip.x || y2 < lowerClip.y || x1 > upperClip.x || y1 > upperClip.y) {
		return false;
	}
	return true;
}

bool Core::Start()
{
    //Quit flag
    bool quit = false;

    //Initialize
    if( Init() == false )
    {
        return 1;
    }

    //Our square object
    //Square square;

    //The frame rate regulator
    //Timer fps;
	float currentFrame = 0;
	float lastFrame = 0;

	float dtmod = 1.0f;

	bool up = false;
	bool down = false;

	this->cameraCenter = Vec2(SCREEN_WIDTH / 2.0f, SCREEN_HEIGHT / 2.0f);
	//this->cameraCenter = Vec2(0,0);
	//glTranslatef(SCREEN_WIDTH / 2.0f, SCREEN_HEIGHT / 2.0f, 0);
	//this->ResetCamera();
	Vec2 camVel = Vec2(0,0);
	float avgFPS = 0;
	int frames = 0;
	float timer = 0.5;

	while(quit == false) {
		currentFrame = (float) SDL_GetTicks();
		float deltaTime = (currentFrame - lastFrame) / 1000;
		lastFrame = currentFrame;
		while( SDL_PollEvent( &event ) )
		{
            //Handle key presses
            //square.handle_input();

			    //If a key was pressed
				if( event.type == SDL_KEYDOWN )
				{
					//Adjust the velocity
					switch( event.key.keysym.sym )
					{
						case SDLK_UP: camVel.y = -1000.0f * 1.0f /cameraZoom; break;
						case SDLK_DOWN: camVel.y = 1000.0f * 1.0f / cameraZoom; break;
						case SDLK_LEFT: camVel.x = -1000.0f * 1.0f / cameraZoom; break;
						case SDLK_RIGHT: camVel.x = 1000.0f * 1.0f / cameraZoom; break;

					}
				}
				//If a key was released
				else if( event.type == SDL_KEYUP )
				{
					//Adjust the velocity
					switch( event.key.keysym.sym )
					{
						case SDLK_UP: camVel.y = 0; break;
						case SDLK_DOWN: camVel.y = 0; break;
						case SDLK_LEFT: camVel.x = 0; break;
						case SDLK_RIGHT: camVel.x = 0; break;
						case SDLK_f: if(paused) TogglePause(); dtmod *= 0.5; break;
						case SDLK_g: if(paused) TogglePause(); dtmod *= 2; break;
						case SDLK_p: TogglePause(); break;
					}
				}
				else if(event.type == SDL_MOUSEBUTTONDOWN) {
                      if (event.button.button == 4)
                        {
                              // glScalef(2, 2, 0);
							   cameraZoom *= 2.0f;
							   //this->SetCamera(cameraCenter * 2);
								//camVel *= 0.5f;				
							   //this->SetCamera(cameraCenter * 2);
                        }
                                                                                                             
					  if (event.button.button == 5) {
                        // glScalef(0.5, 0.5, 0);
							cameraZoom *= 0.5f;
							//camVel *= 2.0f;
							//this->SetCamera(cameraCenter * 0.5);
						}
				}

			if( event.type == SDL_QUIT )
			{
                quit = true;
            }
		}
		glLoadIdentity();
		glScalef(cameraZoom, cameraZoom, 0);

		if(tracking) {
			//this->SetCamera(tracking->pos);
		}
		
		this->SetCamera(cameraCenter + camVel * deltaTime);
		
		timer -= deltaTime;
		frames++;
		if(timer < 0) {
			timer = 0.5f + timer;
			avgFPS = frames / 0.5f;
			frames = 0;
		}
		deltaTime *= dtmod;
		if(paused) deltaTime = 0;
	
		
		GFX::Instance()->GFXDebug("Cam TL: ", GetLowerClip());
		GFX::Instance()->GFXDebug("Cam BR: ", GetUpperClip());
		GFX::Instance()->GFXDebug("FPS: ", avgFPS, 0, 0);
		CollisionManager::UpdateCollision();
		RenderManager::Update(deltaTime);
	    //Move the square
	    //square.move();

	    //Clear the screen
	    //glClear( GL_COLOR_BUFFER_BIT );
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	    //Show the square
	   // square.show();
		RenderManager::Render();
		CollisionManager::DebugDraw();
		/*for(int mayo = 0; mayo < 10000; mayo++) {
			GFX::Instance()->GFXline((float)10000+mayo,(float)10000,(float)11000+mayo,11000, Color(1,1,1));
		}*/

		//Update screen
		gfx->GFXupdate();

		RenderManager::CleanUp();
        //Cap the frame rate
       // if( fps.GetTicks() < 1000 / FRAMES_PER_SECOND )
       // {
        //    SDL_Delay( ( 1000 / FRAMES_PER_SECOND ) - fps.GetTicks() );
        //}
	}

	//Clean up
	SDL_Quit();

	return 0;
}
